/*******************************************************************************
 * University of Illinois/NCSA
 * Open Source License
 *
 * Copyright (c) 2010 , NCSA.  All rights reserved.
 *
 * Developed by:
 * Cyberenvironments and Technologies (CET)
 * http://cet.ncsa.illinois.edu/
 *
 * National Center for Supercomputing Applications (NCSA)
 * http://www.ncsa.illinois.edu/
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal with the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimers.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in the
 *   documentation and/or other materials provided with the distribution.
 * - Neither the names of CET, University of Illinois/NCSA, nor the names
 *   of its contributors may be used to endorse or promote products
 *   derived from this Software without specific prior written permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
 *******************************************************************************/
package edu.uiuc.ncsa.cet.bean.workflow;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import edu.uiuc.ncsa.cet.bean.CETBean;
import edu.uiuc.ncsa.cet.bean.DatasetBean;
import edu.uiuc.ncsa.cet.bean.PersonBean;

public class WorkflowStepBean extends CETBean implements Serializable
{
    /** Used for serialization of object */
    private static final long           serialVersionUID = 1L;

    /** Title of the workflow step */
    private String                      title            = "";                                    //$NON-NLS-1$

    /** creator of the workflow step */
    private PersonBean                  creator          = null;

    /** Date the workflow step is created */
    private Date                        date             = new Date();

    /** Date the workflow step started execution */
    private Date                        start            = null;

    /** Date the workflow step finished execution */
    private Date                        end              = null;

    /** Tool the workflow step is executing */
    private WorkflowToolBean            tool             = null;

    /** Sate of the workflow step */
    private String                      state            = "";                                    //$NON-NLS-1$

    /**
     * List of inputs to the workflow step.
     * 
     */
    private List<DatasetListBean>       inputs           = new ArrayList<DatasetListBean>();

    /**
     * List of outputs to the workflow step.
     * 
     */
    private List<DatasetListBean>       outputs          = new ArrayList<DatasetListBean>();

    /**
     * List of parameters to the workflow step.
     * 
     */
    private List<WorkflowParameterBean> parameters       = new ArrayList<WorkflowParameterBean>();

    /**
     * Create a new instance of the workflow step.
     */
    public WorkflowStepBean()
    {
    }

    /**
     * Return the title of the workflow step.
     * 
     * @return title of the workflow step.
     */
    public String getTitle()
    {
        return title;
    }

    /**
     * Sets the title of the workflow step.
     * 
     * @param title
     *            sets the title of the workflow step.
     * 
     */
    public void setTitle( String title )
    {
        this.title = title;
    }

    /**
     * Return the date when the workflow step was created.
     * 
     * @return date the workflow step was created.
     */
    public Date getDate()
    {
        return date;
    }

    /**
     * Sets the date when the workflow step was created.
     * 
     * @param date
     *            sets the date when the workflow step was created.
     * 
     */
    public void setDate( Date date )
    {
        this.date = date;
    }

    /**
     * Return the date when the workflow step was started.
     * 
     * @return date the workflow step was started.
     */
    public Date getStartDate()
    {
        return start;
    }

    /**
     * Sets the date when the workflow step was started.
     * 
     * @param date
     *            sets the date when the workflow step was started.
     * 
     */
    public void setStartDate( Date date )
    {
        this.start = date;
    }

    /**
     * Return the date when the workflow step was finished.
     * 
     * @return date the workflow step was finished.
     */
    public Date getEndDate()
    {
        return end;
    }

    /**
     * Sets the date when the workflow step was finished.
     * 
     * @param date
     *            sets the date when the workflow step was finished.
     * 
     */
    public void setEndDate( Date date )
    {
        this.end = date;
    }

    /**
     * Return the PersonBean that is the creator of the workflow
     * 
     * @return PersonBean that represents the creator
     */
    public PersonBean getCreator()
    {
        return creator;
    }

    /**
     * Sets the PersonBean that represents the creator of the workflow.
     * 
     * @param creator
     *            sets the PersonBean that represents the creator of the
     *            workflow.
     * 
     */
    public void setCreator( PersonBean creator )
    {
        this.creator = creator;
    }

    /**
     * Return the tool for the workflow step.
     * 
     * @return tool for the workflow step.
     */
    public WorkflowToolBean getTool()
    {
        return tool;
    }

    /**
     * Sets the tool for the workflow step.
     * 
     * @param title
     *            sets the tool for the workflow step.
     * 
     */
    public void setTool( WorkflowToolBean tool )
    {
        this.tool = tool;
    }

    /**
     * Return the state of the workflow step.
     * 
     * @return state of the workflow step.
     */
    public String getState()
    {
        return state;
    }

    /**
     * Sets the state of the workflow step.
     * 
     * @param title
     *            sets the state of the workflow step.
     * 
     */
    public void setState( String state )
    {
        this.state = state;
    }

    /**
     * Return the collection of datasets that are inputs to the step.
     * 
     * @return collection of DatasetListBean that represents all the inputs to
     *         the workflow step.
     */
    public List<DatasetListBean> getInputs()
    {
        return this.inputs;
    }

    /**
     * Set the list of datasets that make up the inputs to the step.
     * 
     * @param inputs
     *            the list of datasets to be used as inputs.
     */
    public void setInputs( List<DatasetListBean> inputs )
    {
        this.inputs.clear();
        if ( inputs != null ) {
            this.inputs.addAll( inputs );
        }
    }

    /**
     * Return the list of datasets associated with this id.
     * 
     * @param id
     *            the id whose list of datasets to return.
     * @return list of datasets associated with the id.
     */
    public DatasetListBean getInput( String id )
    {
        if ( (id == null) || id.equals( "" ) ) { //$NON-NLS-1$
            return null;
        }

        for ( DatasetListBean dlb : inputs ) {
            if ( id.equals( dlb.getId() ) ) {
                return dlb;
            }
        }

        return null;
    }

    /**
     * Add the data to list of inputs to the workflow step.
     * 
     * @param data
     *            the DatasetBean to be added as input.
     */
    public void addInput( DatasetListBean data )
    {
        inputs.add( data );
    }

    /**
     * Remove the data to list from inputs to the workflow step.
     * 
     * @param data
     *            the DatasetBean to be removed as input.
     */
    public void removeInput( DatasetListBean data )
    {
        inputs.remove( data );
    }

    /**
     * Add the data to the right list of inputs to the workflow step.
     * 
     * @param id
     *            the id of the data to be added.
     * @param data
     *            the DatasetBean to be added as input.
     */
    public void addInput( String id, DatasetBean data )
    {
        if ( (id == null) || id.equals( "" ) || (data == null) ) { //$NON-NLS-1$
            return;
        }

        for ( DatasetListBean dlb : inputs ) {
            if ( id.equals( dlb.getId() ) ) {
                dlb.addData( data );
                return;
            }
        }

        DatasetListBean dlb = new DatasetListBean();
        dlb.setId( id );
        dlb.addData( data );
        inputs.add( dlb );
    }

    /**
     * Remove the data from the set of inputs to the worflow step.
     * 
     * @param id
     *            the id of the tooldata to be removed.
     * @param data
     *            the DatasetBean to be removed as input.
     */
    public void removeInput( String id, DatasetBean data )
    {
        if ( (id == null) || id.equals( "" ) || (data == null) ) { //$NON-NLS-1$
            return;
        }

        for ( DatasetListBean dlb : inputs ) {
            if ( id.equals( dlb.getId() ) ) {
                dlb.removeData( data );
                return;
            }
        }
    }

    /**
     * Return the collection of datasets that are outputs to the step.
     * 
     * @return collection of DatasetListBean that represents all the outputs to
     *         the workflow step.
     */
    public List<DatasetListBean> getOutputs()
    {
        return this.outputs;
    }

    /**
     * Set the list of datasets that make up the outputs to the step.
     * 
     * @param outputs
     *            the list of datasets to be used as outputs.
     */
    public void setOutputs( List<DatasetListBean> outputs )
    {
        this.outputs.clear();
        if ( outputs != null ) {
            this.outputs.addAll( outputs );
        }
    }

    /**
     * Return the list of datasets associated with this id.
     * 
     * @param id
     *            the id whose list of datasets to return.
     * @return list of datasets associated with the id.
     */
    public DatasetListBean getOutput( String id )
    {
        if ( (id == null) || id.equals( "" ) ) { //$NON-NLS-1$
            return null;
        }

        for ( DatasetListBean dlb : outputs ) {
            if ( id.equals( dlb.getId() ) ) {
                return dlb;
            }
        }

        return null;
    }

    /**
     * Add the data to list of outputs to the workflow step.
     * 
     * @param data
     *            the DatasetBean to be added as output.
     */
    public void addOutput( DatasetListBean data )
    {
        outputs.add( data );
    }

    /**
     * Remove the data to list from outputs to the workflow step.
     * 
     * @param data
     *            the DatasetBean to be removed as output.
     */
    public void removeOutput( DatasetListBean data )
    {
        outputs.remove( data );
    }

    /**
     * Add the data to the right list of outputs to the workflow step.
     * 
     * @param id
     *            the id of the data to be added.
     * @param data
     *            the DatasetBean to be added as output.
     */
    public void addOutput( String id, DatasetBean data )
    {
        if ( (id == null) || id.equals( "" ) || (data == null) ) { //$NON-NLS-1$
            return;
        }

        for ( DatasetListBean dlb : outputs ) {
            if ( id.equals( dlb.getId() ) ) {
                dlb.addData( data );
                return;
            }
        }

        DatasetListBean dlb = new DatasetListBean();
        dlb.setId( id );
        dlb.addData( data );
        outputs.add( dlb );
    }

    /**
     * Remove the data from the set of outputs to the worflow step.
     * 
     * @param id
     *            the id of the tooldata to be removed.
     * @param data
     *            the DatasetBean to be removed as output.
     */
    public void removeOutput( String id, DatasetBean data )
    {
        if ( (id == null) || id.equals( "" ) || (data == null) ) { //$NON-NLS-1$
            return;
        }

        for ( DatasetListBean dlb : outputs ) {
            if ( id.equals( dlb.getId() ) ) {
                dlb.removeData( data );
                return;
            }
        }
    }

    /**
     * Return the collection of parameters of the step.
     * 
     * @return collection of WorkflowParameterBean that represents all the
     *         parameters to the workflow step.
     */
    public List<WorkflowParameterBean> getParameters()
    {
        return this.parameters;
    }

    /**
     * Sets the collection of parameters of the step.
     * 
     * @param parameters
     *            collection of WorkflowParameterBean that represents all the
     *            parameters to the workflow step.
     */
    public void setParameters( List<WorkflowParameterBean> parameters )
    {
        this.parameters.clear();
        if ( parameters != null ) {
            this.parameters.addAll( parameters );
        }
    }

    /**
     * Add the parameter from the set of parameters to the worflow step.
     * 
     * @param parameter
     *            the parameter to be added.
     */
    public void addParameter( WorkflowParameterBean parameter )
    {
        this.parameters.add( parameter );
    }

    /**
     * Remove the parameter from the set of parameters to the worflow step.
     * 
     * @param parameter
     *            the parameter to be removed.
     */
    public void removeParameter( WorkflowParameterBean parameter )
    {
        this.parameters.remove( parameter );
    }
}
